

-- map gen settings for the different planets
local nauvis_map = data.raw.planet.nauvis.map_gen_settings
local nauvis_original = table.deepcopy(data.raw.planet.nauvis.map_gen_settings)
local vulcanus_map = data.raw.planet.vulcanus.map_gen_settings
local gleba_map = data.raw.planet.gleba.map_gen_settings
local fulgora_map = data.raw.planet.fulgora.map_gen_settings
local aquilo_map = data.raw.planet.aquilo.map_gen_settings


-- custom noise 
data:extend{
    {
        type = "noise-expression",
        name = "noise_constant",
        expression = "600"
    },
    {
        type = "noise-function",
        name = "noise_multiply",
        parameters = {"noise_1", "noise_2"},
        expression = "(noise_1)*(noise_2)"
    },
    {
        type = "noise-function",
        name = "noise_add",
        parameters = {"noise_1", "noise_2"},
        expression = "(noise_1)+(noise_2)"
    },
    {
        type = "noise-expression",
        name = "nauvis_belt",
        expression = "(if(y<=noise_constant, 1, 0))*(if(y>-noise_constant, 1, 0))"
    },
    {
        type = "noise-expression",
        name = "vulcanus_belt",
        expression = "(if(y<=2*noise_constant, 1, 0))*(if(y>noise_constant, 1, 0))"
    },
    {
        type = "noise-expression",
        name = "aquilo_belt",
        expression = "if(y>2*noise_constant, 1, 0)"
    },
    {
        type = "noise-expression",
        name = "fulgora_belt",
        expression = "(if(y<=-noise_constant, 1, 0))*(if(y>-2*noise_constant, 1, 0))"
    },
    {
        type = "noise-expression",
        name = "gleba_belt",
        expression = "if(y<=-2*noise_constant, 1, 0)"
    },
}

local noise_table = {
    ["nauvis"] = "nauvis_belt",
    ["vulcanus"] = "vulcanus_belt",
    ["fulgora"] = "fulgora_belt",
    ["gleba"] = "gleba_belt",
    ["aquilo"] = "aquilo_belt"
}

-- function to add planet_1 map settings to planet_2 map settings
local function add_controls(map_1, map_2)
    for index, aa in pairs(map_2.autoplace_controls) do
        map_1.autoplace_controls[index] = {}
    end
end

-- function to add planet_1 autoplace_controls to planet_2 
local function add_settings(map_1, map_2)
    local settings_list = {"tile", "decorative", "entity"}
    for _, field in pairs(settings_list) do
        for index, aa in pairs(map_2.autoplace_settings[field].settings) do
            map_1.autoplace_settings[field].settings[index] = {}
        end
    end
end

-- function to automatically change a new expression
local function multiply_noise(noise, planet)
    return "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..noise.."}"
end

-- function to evaluate if a prototype is included in a table
local function has_value(tab, val)
    for index, value in pairs(tab) do
        if index == val or value==val then
            return true
        end
    end
    return false
end

-- function to evaluate if a prototype is included in a map_gen
local function is_on_map(map_gen, prototype)
    local settings_list = {"tile", "decorative", "entity"}
    for _, field in pairs(settings_list) do
        local table = map_gen.autoplace_settings[field].settings
        if has_value(table, prototype.name) then
            return true
        end
    end
    return false
end

-- function return a table including a list of planets on which prototype is included
local function planet_list(prototype)
    local list = {}
    if is_on_map(nauvis_original, prototype) then
        table.insert(list, 1, "nauvis")
    end
    if is_on_map(vulcanus_map, prototype) then
        table.insert(list, 1, "vulcanus")
    end
    if is_on_map(fulgora_map, prototype) then
        table.insert(list, 1, "fulgora")
    end
    if is_on_map(gleba_map, prototype) then
        table.insert(list, 1, "gleba")
    end
    if is_on_map(aquilo_map, prototype) then
        table.insert(list, 1, "aquilo")
    end
    return list
end

-- add settings autoplace from other planets to nauvis
add_settings(nauvis_map, vulcanus_map)
add_settings(nauvis_map, gleba_map)
add_settings(nauvis_map, fulgora_map)
add_settings(nauvis_map, aquilo_map)

-- add controls autoplace from other planets to nauvis
add_controls(nauvis_map, vulcanus_map)
add_controls(nauvis_map, gleba_map)
add_controls(nauvis_map, fulgora_map)
add_controls(nauvis_map, aquilo_map)

-- Sum together all elevation expressions....not working yet
local nauvis_elevation_limited = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..data.raw["noise-expression"]["elevation_nauvis"]["expression"].."}"
local vulcanus_elevation_limited = "noise_multiply{noise_1 ="..noise_table["vulcanus"]..", noise_2 = "..data.raw["noise-expression"]["vulcanus_elevation"]["expression"].."}"
local fulgora_elevation_limited = "noise_multiply{noise_1 ="..noise_table["fulgora"]..", noise_2 = "..data.raw["noise-expression"]["fulgora_elevation"]["expression"].."}"
local gleba_elevation_limited = "0" --"noise_multiply{noise_1 ="..noise_table["gleba"]..", noise_2 = "..data.raw["noise-expression"]["gleba_elevation"]["expression"].."}"
local aquilo_elevation_limited = "0"--"noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..data.raw["noise-expression"]["aquilo_elevation"]["expression"].."}"
local new_elevation = "("..nauvis_elevation_limited..")+("..vulcanus_elevation_limited..")+("..fulgora_elevation_limited..")+("..gleba_elevation_limited..")+("..aquilo_elevation_limited..")"
--data.raw.planet.nauvis.map_gen_settings.property_expression_names["elevation"] = data.raw["noise-expression"]["aquilo_elevation"]["expression"]
--data.raw["noise-expression"]["elevation_nauvis"]["expression"] = new_elevation

-- list of fields containing relevant prototype
local data_fields = {
    "tree",
    "simple-entity",
    "resource",
    "tile",
    "optimized-decorative"
}

-- Lists of planets trees
local nauvis_trees = {
    "tree-01",
    "tree-02",
    "tree-02-red",
    "tree-03",
    "tree-04",
    "tree-05",
    "tree-06",
    "tree-06-brown",
    "tree-07",
    "tree-08-brown",
    "tree-08-red",
    "tree-09",
    "tree-09-brown",
    "tree-09-red",
    "dead-dry-hairy-tree",
    "dead-grey-trunk",
    "dead-tree-desert",
    "dry-hairy-tree",
    "dry-tree",
}

local vulcanus_trees = {
    "ashland-lichen-tree",
    "ashland-lichen-tree-flaming",
}

local gleba_trees = {
    "boompuff",
    "cuttlepop",
    "hairyclubnub",
    "lickmaw",
    "teflilly",
    "funneltrunk",
    "slipstack",
    "stingfrond",
    "sunnycomb",
    "water-cane",
}

local function check_tree(name)
    return has_value(gleba_trees, name) or has_value(vulcanus_trees, name)
end

-- Change the tree autoplacing
for _, name in pairs(gleba_trees) do
    local expression = data.raw["tree"][name].autoplace.probability_expression
    data.raw["tree"][name].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["gleba"]..", noise_2 = "..expression.."}"
end
for _, name in pairs(vulcanus_trees) do
    local expression = data.raw["tree"][name].autoplace.probability_expression
    data.raw["tree"][name].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["vulcanus"]..", noise_2 = "..expression.."}"
end
for _, name in pairs(nauvis_trees) do
    local expression = data.raw["tree"][name].autoplace.probability_expression
    data.raw["tree"][name].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..expression.."}"
end
-- there are additional trees from the nauvis biomes which are not caugth by the above loop and must be brute-forced
for _, prototype in pairs(data.raw["tree"]) do  
    if prototype.autoplace and not check_tree(prototype.name) then
        local expression = prototype.autoplace.probability_expression
        prototype.autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..expression.."}"
    end
end

-- Change the enemy base spawn probability
local nauvis_enemy_base = data.raw["noise-expression"]["enemy_base_probability"].expression
data.raw["noise-expression"]["enemy_base_probability"].expression = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..nauvis_enemy_base.."}"

-- Demolishers
data.raw.planet.nauvis.map_gen_settings.territory_settings = data.raw.planet.vulcanus.map_gen_settings.territory_settings
data:extend{
    {
        type = "noise-expression",
        name = "extended_safe_zone",
        expression = noise_table["nauvis"].."*100+"..noise_table["fulgora"].."*100+"..noise_table["gleba"].."*100+"..noise_table["aquilo"].."*100"
    },
}
local demolisher = data.raw["noise-expression"]["demolisher_territory_expression"].expression
data.raw["noise-expression"]["demolisher_territory_expression"].expression = demolisher.."-extended_safe_zone"

-- Gleba enemies
local gleba_spawner = data.raw["noise-expression"]["gleba_spawner"].expression
data.raw["noise-expression"]["gleba_spawner"].expression = "noise_multiply{noise_1 ="..noise_table["gleba"]..", noise_2 = "..gleba_spawner.."}"
local gleba_spawner = data.raw["noise-expression"]["gleba_spawner_small"].expression
data.raw["noise-expression"]["gleba_spawner_small"].expression = "noise_multiply{noise_1 ="..noise_table["gleba"]..", noise_2 = "..gleba_spawner.."}"

-- Change the tiles autoplace settings
for _, prototype in pairs(data.raw["tile"]) do
    if prototype.autoplace and planet_list(prototype)~={} then
        --local old_expression = prototype.autoplace.probability_expression
        --local new_expression = ""
        for _, planet in pairs(planet_list(prototype)) do
            --local temp_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..old_expression.."}"
            --new_expression = "noise_add{noise_1 = "..new_expression..", noise_2= "..temp_expression.."}"
            local expression = prototype.autoplace.probability_expression
            prototype.autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..expression.."}"
        end
    end
end

-- Change the 
--local ammonia_old = data.raw["noise-expression"]["ammoniacal_ocean"].expression
--data.raw["noise-expression"]["ammoniacal_ocean"].expression = "noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..ammonia_old.."}"
--local ammonia_old = data.raw["noise-expression"]["ammoniacal_ocean_2"].expression
--data.raw["noise-expression"]["ammoniacal_ocean_2"].expression = "noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..ammonia_old.."}"
local ammonia_old = data.raw["tile"]["ammoniacal-ocean"].autoplace.probability_expression
data.raw["tile"]["ammoniacal-ocean"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..ammonia_old.."}"
local ammonia_old = data.raw["tile"]["ammoniacal-ocean-2"].autoplace.probability_expression
data.raw["tile"]["ammoniacal-ocean-2"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..ammonia_old.."}"

-- Change the tiles autoplace settings
for _, prototype in pairs(data.raw["optimized-decorative"]) do
    if prototype.autoplace and planet_list(prototype) then
        --local old_expression = prototype.autoplace.probability_expression
        --local new_expression = ""
        for _, planet in pairs(planet_list(prototype)) do
            --local temp_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..old_expression.."}"
            --new_expression = "noise_add{noise_1 = "..new_expression..", noise_2= "..temp_expression.."}"
            local expression = prototype.autoplace.probability_expression
            prototype.autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..expression.."}"
        end
    end
end

-- Change the entities autoplace settings
for _, prototype in pairs(data.raw["simple-entity"]) do
    if prototype.autoplace and planet_list(prototype) then
        --local old_expression = prototype.autoplace.probability_expression
        --local new_expression = ""
        for _, planet in pairs(planet_list(prototype)) do
            --local temp_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..old_expression.."}"
            --new_expression = "noise_add{noise_1 = "..new_expression..", noise_2= "..temp_expression.."}"
            local expression = prototype.autoplace.probability_expression
            prototype.autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table[planet]..", noise_2 = "..expression.."}"
        end
    end
end

-- change fish spawning
local expression = data.raw["fish"]["fish"].autoplace.probability_expression
data.raw["fish"]["fish"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..expression.."}"
-- change lighting rods spawning
local expression = data.raw["lightning-attractor"]["fulgoran-ruin-attractor"].autoplace.probability_expression
data.raw["lightning-attractor"]["fulgoran-ruin-attractor"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["fulgora"]..", noise_2 = "..expression.."}"

-- Change the resources autoplace settings
for _, prototype in pairs(data.raw["resource"]) do
    if prototype.autoplace and is_on_map(nauvis_original,prototype) then
        local expression = prototype.autoplace.probability_expression
        prototype.autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["nauvis"]..", noise_2 = "..expression.."}"
    end
end

local expression = data.raw["resource"]["scrap"].autoplace.probability_expression
data.raw["resource"]["scrap"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["fulgora"]..", noise_2 = "..expression.."}"

local expression = data.raw["resource"]["fluorine-vent"].autoplace.probability_expression
data.raw["resource"]["fluorine-vent"].autoplace.probability_expression = "noise_multiply{noise_1 ="..noise_table["aquilo"]..", noise_2 = "..expression.."}"

local function change_probability(planet_name, prototype)
    prototype = prototype:gsub("-","_") -- format name to be underscore by default
    local target = planet_name.."_"..prototype
    local prob = target.."_probability"
    local rich = target.."_richness"
    data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:"..prototype:gsub("_","-")..":probability"] = prob
    data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:"..prototype:gsub("_","-")..":richness"] = rich
    local old_expression = data.raw["noise-expression"][prob].expression
    data.raw["noise-expression"][prob].expression = "noise_multiply{noise_1 ="..noise_table[planet_name]..", noise_2 = "..old_expression.."}"
end

-- vulcanus resources fix 
change_probability("vulcanus","calcite")
change_probability("vulcanus","tungsten-ore")
change_probability("vulcanus","sulfuric-acid-geyser")
-- aquilo resources fix 
change_probability("aquilo", "lithium-brine")



--[[

data:extend{
    {
        type = "noise-function",
        name = "noise_multiply",
        parameters = {"noise_1", "noise_2"},
        expression = "(noise_1)*(noise_2)"
    },
    {
        type = "noise-expression",
        name = "nauvis_belt",
        --expression = "0"
        expression = "if(y>=10, 0, 1)"
    },
}


local function has_value (tab, val)
    for index, value in pairs(tab) do
        if index == val then
            return true
        end
    end

    return false
end


local data_fields = {
    "tree",
    "simple-entity",
    "resource",
    "tile",
    "optimized-decorative"
}

for _, field in ipairs(data_fields) do
    for _, prototype in pairs(data.raw[field]) do
        if prototype.autoplace and has_value(data.raw.planet.nauvis.map_gen_settings.autoplace_settings["entity"].settings, prototype.name) then
            local expression = prototype.autoplace.probability_expression
            prototype.autoplace.probability_expression = "noise_multiply{noise_1 = nauvis_belt, noise_2 = "..expression.."}"
        end
    end
end

data.raw.planet.nauvis.map_gen_settings.autoplace_controls["calcite"] = {}
data.raw.planet.nauvis.map_gen_settings.autoplace_settings.entity.settings["calcite"] = {}
data.raw.planet.nauvis.map_gen_settings.autoplace_controls["tungsten_ore"] = {}
data.raw.planet.nauvis.map_gen_settings.autoplace_settings.entity.settings["tungsten-ore"] = {}
data.raw.planet.nauvis.map_gen_settings.autoplace_controls["vulcanus_volcanism"] = {}
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:tungsten-ore:probability"] = "vulcanus_tungsten_ore_probability"
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:tungsten-ore:richness"] = "vulcanus_tungsten_ore_richness"
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:calcite:probability"] = "vulcanus_calcite_probability"
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:calcite:richness"] = "vulcanus_calcite_richness"
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:sulfuric-acid-geyser:probability"] = "vulcanus_sulfuric_acid_geyser_probability"
data.raw.planet.nauvis.map_gen_settings.property_expression_names["entity:sulfuric-acid-geyser:richness"] = "vulcanus_sulfuric_acid_geyser_richness"

--]]
